#! /usr/bin/ruby

# Copyright (C) 2010 Jérémy Pages.
 
# This file is part of Mnemochess.
 
# Mnemochess is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
 
# Mnemochess is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
 
# You should have received a copy of the GNU General Public License
# along with Mnemochess.  If not, see <http://www.gnu.org/licenses/>.

require 'gtk2'
require 'board.rb'
require 'game.rb'

PIECENAMES = ['king','queen','rook','bishop','knight','pawn']
LEFT_BUTTON = 1
RIGHT_BUTTON = 3

class MnemochessUI < Gtk::Window
	attr_accessor :leftBoard, :rightBoard, :pieceButtons, :menuBar, :score,
			:chrono, :game

	def initialize
		super("Mnemochess 0.1")
		self.set_resizable(false)
		
		@menuBar = Gtk::MenuBar.new()
		@leftBoard = Board.new()
		@rightBoard = Board.new()
		@pieceButtons = Array.new(12) { |i| Gtk::Button.new } 
		@score = Gtk::Label.new("Score : 0 / 0")
		@chrono = Gtk::Label.new("Time : 0:00")
		@game = Game.new()

		accelgroup = Gtk::AccelGroup.new

		widgetstable = Gtk::Table.new(3,2,false)
		piecetable = Gtk::Table.new(2,6,true)
		labelbox = Gtk::VBox.new()
		labelbox.pack_start(@score)
		labelbox.pack_start(@chrono)
		self.add(widgetstable)
		widgetstable.attach_defaults(@menuBar,0,2,0,1)
		widgetstable.attach_defaults(@leftBoard,0,1,1,2)
		widgetstable.attach_defaults(@rightBoard,1,2,1,2)
		widgetstable.attach_defaults(labelbox,0,1,2,3)
		widgetstable.attach_defaults(piecetable,1,2,2,3)
		
		gamemenu = Gtk::Menu.new
		helpmenu = Gtk::Menu.new
		game = Gtk::MenuItem.new("Game")
		help = Gtk::MenuItem.new("Help")
		game.submenu = gamemenu
		help.submenu = helpmenu	
		@menuBar.append(game)
		@menuBar.append(help)

		newGame = Gtk::ImageMenuItem.new(Gtk::Stock::NEW,accelgroup)
		quit = Gtk::ImageMenuItem.new(Gtk::Stock::QUIT,accelgroup)
		about = Gtk::ImageMenuItem.new(Gtk::Stock::ABOUT,accelgroup)
		gamemenu.append(newGame)
		gamemenu.append(quit)
		helpmenu.append(about)		

		for i in 0..5
			pieceicon = Gtk::Image.new('../img/white_'+PIECENAMES[i]+'.png')
			@pieceButtons[i].set_image(pieceicon)
			piecetable.attach_defaults(@pieceButtons[i],i,i+1,0,1)
			pieceicon = Gtk::Image.new('../img/black_'+PIECENAMES[i]+'.png')
			@pieceButtons[6+i].set_image(pieceicon)
			piecetable.attach_defaults(@pieceButtons[6+i],i,i+1,1,2)
		end
		
		self.signal_connect('destroy') do Gtk.main_quit end
		@pieceButtons[0].signal_connect('clicked') do @selectedpiece = 'K' end
		@pieceButtons[1].signal_connect('clicked') do @selectedpiece = 'Q' end
		@pieceButtons[2].signal_connect('clicked') do @selectedpiece = 'R' end
		@pieceButtons[3].signal_connect('clicked') do @selectedpiece = 'B' end
		@pieceButtons[4].signal_connect('clicked') do @selectedpiece = 'N' end
		@pieceButtons[5].signal_connect('clicked') do @selectedpiece = 'P' end
		@pieceButtons[6].signal_connect('clicked') do @selectedpiece = 'k' end
		@pieceButtons[7].signal_connect('clicked') do @selectedpiece = 'q' end
		@pieceButtons[8].signal_connect('clicked') do @selectedpiece = 'r' end
		@pieceButtons[9].signal_connect('clicked') do @selectedpiece = 'b' end
		@pieceButtons[10].signal_connect('clicked') do @selectedpiece = 'n' end
		@pieceButtons[11].signal_connect('clicked') do @selectedpiece = 'p' end
		newGame.signal_connect('activate') do
			#@game = Game::new
			puts 'New game'
		end
		quit.signal_connect('activate') do Gtk.main_quit end
		about.signal_connect('activate') do
			aboutdialog = Gtk::AboutDialog.new
			aboutdialog.set_name('Mnemochess')
			aboutdialog.show
		end

		diag = @game.rightDiagram
		@rightBoard.set_events(Gdk::Event::BUTTON_PRESS_MASK)
		@rightBoard.signal_connect('button_press_event'){|w,e| click_on_board(@rightBoard,diag,@selectedpiece,e)}		

		self.show_all
	end

	def click_on_board(board, diag, piece, event)
		x = event.x
		y = event.y
		button = event.button

		# We must now determine which square was selected 
		column = ((x-30)/30).floor
		rank = ((270-y)/30).floor
		puts "column=#{column} rank=#{rank} button=#{button}"
		
		if column >= 0 and column <=7 and rank >=0 and rank <= 7 and piece != nil
			case button
				when LEFT_BUTTON
					board.set_square(column,rank,piece)
					diag.set_square(column,rank,piece)
				when RIGHT_BUTTON
					board.set_square(column,rank,'.')
					diag.set_square(column,rank,'.')
			end
		end
	end

	def mainloop
		Gtk.main
	end
end
